<?php

namespace Drupal\instana_eum\Form;

use Drupal\Core\Config\Config;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Markup;

/**
 * Instana settings form.
 */
class InstanaConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'instana_eum_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['instana_eum.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(
    array $form,
    FormStateInterface $form_state,
  ): array {

    $config = $this->config('instana_eum.settings');

    /* SETTINGS FORM */
    $form['settings'] = [
      '#type' => 'vertical_tabs',
    ];

    $form['general_settings'] = $this->getGeneralSettings($config);
    $form['general_settings']['#open'] = TRUE;

    $form['adv_settings'] = $this->getAdvancedSettings($config);

    return parent::buildForm($form, $form_state);

  }

  /**
   * Generates the part of the form: General Settings.
   *
   * @param \Drupal\Core\Config\Config $config
   *   Module configuration.
   *
   * @return array
   *   Array with content to be rendered, if applicable.
   */
  private function getGeneralSettings(Config $config): array {
    $form = [];

    $form['general_settings'] = [
      '#type'        => 'details',
      '#title'       => $this->t('Connection settings'),
      '#group'       => 'settings',
      '#description' => $this->t('Configures Instana End User Monitoring (EUM) beacon module. Please see the <a href="https://www.instana.com/docs/website_monitoring/" target="_blank">official documentation</a> for further details.'),

      'enabled' => [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Activate Instana EUM'),
        '#description'   => $this->t("Activate or deactivate Instana EUM without having to uninstall the module."),
        '#default_value' => $config->get('enabled') ?? FALSE,
      ],
      'api_key' => [
        '#type'          => 'password',
        '#default_value' => $config->get('api_key') ?? '',
        '#title'         => $this->t('API Key'),
        '#description'   => $this->t("Enter the API key for your Instana server."),
        '#required'      => TRUE,
        '#attributes'    => ['value' => $config->get('api_key')],
      ],
      'reporting_url' => [
        '#type'          => 'textfield',
        '#title'         => $this->t('Reporting URL'),
        '#description'   => $this->t("Enter the Instana server reporting URL."),
        '#default_value' => $config->get('reporting_url') ?? '',
        '#required'      => TRUE,
      ],
    ];

    return $form['general_settings'];
  }

  /**
   * Generates the part of the form: Advanced Settings.
   *
   * @param \Drupal\Core\Config\Config $config
   *   Module configuration.
   *
   * @return array
   *   Array with content to be rendered, if applicable.
   */
  private function getAdvancedSettings(Config $config): array {
    $form = [];

    $form['adv_settings'] = [
      '#type'        => 'details',
      '#title'       => $this->t('Advanced settings'),
      '#group'       => 'settings',

      'track_pages' => [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Track individual pages'),
        '#description'   => $this->t("Isolate specific pages and analyze their performance to find pages with the most traffic, or the slowest response times."),
        '#default_value' => $config->get('track_pages') ?? FALSE,
      ],
      'track_admin' => [
        '#type'          => 'checkbox',
        '#title'         => $this->t('Admin page tracking'),
        '#description'   => $this->t("Include monitoring of pages with /admin in the url."),
        '#default_value' => $config->get('track_admin') ?? FALSE,
      ],
      'advanced_settings' => [
        '#type'          => 'textarea',
        '#default_value' => $config->get('advanced_settings') ?? '',
        '#title'         => $this->t('Advanced Settings'),
        '#description'   => $this->t('You can use this to set additional EUM settings. @url. <p>Examples:</p>@example1 @example2', [
          '@url'      => Markup::create('<a href="https://www.instana.com/docs/website_monitoring/api/" target="_blank">' . $this->t('See documentation here') . '</a>'),
          '@example1' => Markup::create('<pre>ineum(\'meta\', \'version\', \'1.42.3\');</pre>'),
          '@example2' => Markup::create('<pre>ineum(\'ignoreUrls\', [/.*\/api\/data.*/]);</pre>'),
        ]),
      ],
    ];

    return $form['adv_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(
    array &$form,
    FormStateInterface $form_state,
  ): void {
    $config = $this->config('instana_eum.settings');

    /* Configuration fields to be saved. */
    $list = [
      'enabled',
      'api_key',
      'reporting_url',
      'track_pages',
      'track_admin',
      'advanced_settings',
    ];

    foreach ($list as $item) {
      $config->set($item, $form_state->getValue($item));
    }
    $config->save();

    parent::submitForm($form, $form_state);

  }

}
